/*******************************************************************
*
*  DESCRIPTION: class Analizer
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 26/04/2003
*
*******************************************************************/

#ifndef __ANALIZER_H
#define __ANALIZER_H

#include <map.h>
#include "atomic.h"
#include "message.h"

class Analizer: public Atomic
{
public:
    Analizer (const string &name = "Analizer");     //Default constructor

    virtual string className () const ;

protected:
    Model &initFunction ();
    Model &externalFunction (const ExternalMessage &);
    Model &internalFunction (const InternalMessage &);
    Model &outputFunction (const InternalMessage &);

private:
    typedef map < int, Time, less<int> > JobsList ;

    // Input Ports
    const Port &arrived;
    const Port &solved;
    const Port &stop;

    // Output Ports
    Port &average;
    Port &quantity;

    Time totElapsed;
    long arrivedCount;
    long solvedCount;
    bool hasToStop;

    JobsList unsolved; 

};  // class Analizer

// ** inline ** // 
inline
string Analizer::className () const {
    return "Analizer";
}

#endif   //__ANALIZER_H 
