/*******************************************************************
*
*  DESCRIPTION: class CarControlUnit
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 02/05/2003
*
*******************************************************************/

/** include files **/
#include "car_cu.h"       // base header
#include "message.h"       // class InternalMessage 
#include "mainsimu.h"      // class Simulator
#include "distri.h"        // class Distribution 
#include "strutil.h"       // str2Value( ... )

/*******************************************************************
* Function Name: CarControlUnit
********************************************************************/
CarControlUnit::CarControlUnit (const string &name)
: Atomic (name)
, in (addInputPort ("in"))
, out (addInputPort ("out")) 
, opens (addOutputPort ("opens")) 
, openn (addOutputPort ("openn")) {
	if (MainSimulator::Instance ().existsParameter (description (), "max_north")) {
		max_north = str2Int (MainSimulator::Instance ().getParameter (description (), "max_north"));
	} else {
		max_north = 1;
	}
	if (MainSimulator::Instance ().existsParameter (description (), "max_south")) {
		max_south = str2Int (MainSimulator::Instance ().getParameter (description (), "max_south"));
	} else {
		max_south = 1;
	}
	if (MainSimulator::Instance ().existsParameter (description (), "timeout")) {
		timeout = MainSimulator::Instance ().getParameter (description (), "timeout");
	} else {
		// por default, 1 minuto.
		timeout = Time (0, 1, 0, 0);
	}
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &CarControlUnit::initFunction () {
    way = NORTH;
    total = in_transit = 0;
    phase = OPEN_NORTH;
    holdIn (active, Time::Zero);
    return *this;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &CarControlUnit::externalFunction (const ExternalMessage &msg) {
    if (msg.port () == in) {
        total++;
        in_transit++;
        if (way == NORTH) {
        	if (total == max_north) {
            	phase = CLOSE_NORTH;
            	holdIn (active, Time::Zero);
            } else {
            	phase = NONE;
            	holdIn (active, timeout);
            }
        }
        if (way == SOUTH) {
        	if (total == max_south) {
            	phase = CLOSE_SOUTH;
            	holdIn (active, Time::Zero);
            } else {
            	phase = NONE;
            	holdIn (active, timeout);
            }
        }
    }
    if (msg.port () == out) {
        in_transit--;
        if (in_transit == 0) {
            if (way == NORTH) {
            	// solo cambio de lado si pasaron todos los vehiculos.
            	if (total == max_north) {
		            total = 0;
        	        way = SOUTH;
    	            phase = OPEN_SOUTH;
            	    holdIn (active, Time::Zero);
            	}
            } else {
            	// solo cambio de lado si pasaron todos los vehiculos.
            	if (total == max_south) {
		            total = 0;
    	            way = NORTH;
	                phase = OPEN_NORTH;
        	        holdIn (active, Time::Zero);
        	    }
            }   
        }
    }
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &CarControlUnit::internalFunction (const InternalMessage &msg) {
	if (phase == NONE) {
		// vengo de un timeout..., con lo cual, tengo que cambiar 
		// de sentido....
		way = (way == NORTH ? SOUTH : NORTH);
	} else {	
		// vengo de un open o close, con lo cual, scheduleo el proximo
		// timeout...
   		phase = NONE;
	}	
	holdIn (active, timeout);
    return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &CarControlUnit::outputFunction (const InternalMessage &msg) {
    switch (phase) {
        case OPEN_NORTH:
            sendOutput (msg.time (), openn, 1);
            break;
        case OPEN_SOUTH:
            sendOutput (msg.time (), opens, 1);
            break;
        case CLOSE_NORTH:
            sendOutput (msg.time (), openn, 0);
            break;
        case CLOSE_SOUTH:
            sendOutput (msg.time (), opens, 0);
            break;
        case NONE:
        	if (way == NORTH) {
	            sendOutput (msg.time (), openn, 0);
	            sendOutput (msg.time (), opens, 1);
        	} else {
	            sendOutput (msg.time (), opens, 0);
	            sendOutput (msg.time (), openn, 1);
        	}	
            break;
    }
    return *this ;
}

