/*******************************************************************
*
*  DESCRIPTION: class Distributor
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 01/05/2003
*
*******************************************************************/

/** include files **/
#include "distributor.h"   // base header
#include "message.h"       // class InternalMessage 
#include "mainsimu.h"      // class Simulator
#include "distri.h"        // class Distribution 
#include "strutil.h"       // str2Value( ... )

/*******************************************************************
* Function Name: Distributor
********************************************************************/
Distributor::Distributor (const string &name)
: Atomic (name)
, opens (addInputPort ("opens"))
, openn (addInputPort ("openn"))
, in (addInputPort ("in"))
, done (addInputPort ("done"))
, outs (addOutputPort ("outs")) 
, outn (addOutputPort ("outn")) 
, out_tra (addOutputPort ("out_tra")) {
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Distributor::initFunction () {
    // Lo unico importante es que el semaphoro este cerrado para que
    // no deje pasar coches hasta no recibir un open.
    id_in = id_done = 0;
    way = UNKNOWN;
    phase = NONE;
    signal_open = false;
    passivate ();
    return *this;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &Distributor::externalFunction (const ExternalMessage &msg) {
    if (msg.port () == opens) {
       way = TO_NORTH;
       signal_open = msg.value ();     
    }
    if (msg.port () == openn) {
       way = TO_SOUTH;
       signal_open = msg.value ();     
    }
    if (msg.port () == in) {
        // solo dejo entrar si el semaforo esta abierto.
        if (signal_open) {
            id_in = static_cast< int > (msg.value ());
            phase = TRANSIT;
            holdIn (active, Time::Zero);
        }
    }
    if (msg.port () == done) {
        // como estos son los que estan en transito, siempre los dejo salir.
        id_done = static_cast< int > (msg.value ());
        switch (way) {
            case TO_NORTH: 
                phase = NORTH;
                break;
            case TO_SOUTH: 
                phase = SOUTH;
                break;
            default: 
                phase = NONE;
                break;
        }
        holdIn (active, Time::Zero);
    }
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Distributor::internalFunction (const InternalMessage &) {
    passivate ();
    return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Distributor::outputFunction (const InternalMessage &msg) {
    switch (phase) {
        case NORTH:
            sendOutput (msg.time (), outn, id_done);
            break;
        case SOUTH:
            sendOutput (msg.time (), outs, id_done);
            break;
        case TRANSIT:
            sendOutput (msg.time (), out_tra, id_in);
            break;
        case NONE:
            break;
    }
    // reset flag.
    phase = NONE;
    return *this ;
}

