/*******************************************************************
*
*  DESCRIPTION: class Queue
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 04/05/2003
*
*******************************************************************/

/** include files **/
#include "queue.h"      // class Queue
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

/** public functions **/

/*******************************************************************
* Function Name: Queue
* Description: 
********************************************************************/
Queue::Queue (const string &name)
: Atomic (name)
, in (addInputPort ("in"))
, start (addInputPort ("start"))
, out (addOutputPort ("out")) {
	if (MainSimulator::Instance ().existsParameter (description (), "preparation")) {
		preparationTime = MainSimulator::Instance ().getParameter (description (), "preparation");
	} else {
		// por default, 10 segundos.
		preparationTime = Time (0, 0, 10, 0);
	}
}

/*******************************************************************
* Function Name: initFunction
* Description: Resetea la lista
* Precondition: El tiempo del proximo evento interno es Infinito
********************************************************************/
Model &Queue::initFunction () {
	elements.erase (elements.begin (), elements.end ());
	passivate ();
	return *this;
}

/*******************************************************************
* Function Name: externalFunction
* Description: 
********************************************************************/
Model &Queue::externalFunction (const ExternalMessage &msg) {
	if (msg.port () == in) 	{
		elements.push_back (msg.value ());
		if (state () == active && elements.size () == 1) {
			holdIn (active, Time::Zero);
		}
	}

	if (msg.port () == start) {
		if (state () == passive && msg.value ()) {
			holdIn (active, Time::Zero);
		} else {
			if (state () == active && !msg.value ()) {
				passivate ();
			}
		}
	}
	return *this;
}

/*******************************************************************
* Function Name: internalFunction
* Description: 
********************************************************************/
Model &Queue::internalFunction (const InternalMessage &msg) {
	holdIn (active, preparationTime);
	return *this ;
}

/*******************************************************************
* Function Name: outputFunction
* Description: 
********************************************************************/
Model &Queue::outputFunction (const InternalMessage &msg) {
	if (!elements.empty ()) {
		sendOutput (msg.time(), out, elements.front ());
		elements.pop_front ();
	}
	return *this ;
}
