/*******************************************************************
*
*  DESCRIPTION: class Traffic
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 26/04/2003
*
*******************************************************************/

/** include files **/
#include "traffic.h"       // base header
#include "message.h"       // class InternalMessage 
#include "mainsimu.h"      // class Simulator
#include "distri.h"        // class Distribution 
#include "strutil.h"       // str2Value( ... )

/*******************************************************************
* Function Name: Traffic
********************************************************************/
Traffic::Traffic (const string &name)
: Atomic (name)
, stop (addInputPort ("stop"))
, out (addOutputPort ("out")) {
    try {
        dist = Distribution::create (MainSimulator::Instance ().getParameter (description (), "distribution"));
        MASSERT (dist);
        for (register int i = 0; i < dist->varCount (); i++) {
            string parameter (MainSimulator::Instance ().getParameter (description (), dist->getVar (i)));
            dist->setVar (i, str2Value (parameter));
        }
        initial = 1;
        increment = 1;
    } catch (InvalidDistribution &e) {
        e.addText ("The model " + description () + " has distribution problems!" );
        e.print (cerr);
        MTHROW (e) ;
    } catch (MException &e) {
        MTHROW (e);
    }
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Traffic::initFunction () {
    pid = initial;
    holdIn (active, Time::Zero);
    return *this;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &Traffic::externalFunction (const ExternalMessage &msg) {
    // Si recibo un STOP, lo apago para siempre....
    if (msg.port () == stop) {
        passivate ();
    }
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Traffic::internalFunction (const InternalMessage &) {
    // Solo sigo generando trafico si estoy activo, porque si estoy
    // pasivo, significa que ya recibi un STOP.
    if (state () == active) {
        holdIn (active, Time (static_cast< float > (fabs (distribution ().get ()))));
    }
    return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Traffic::outputFunction (const InternalMessage &msg) {
    sendOutput (msg.time (), out, pid) ;
    pid += increment;
    return *this ;
}

Traffic::~Traffic () {
    delete dist;
}
