/*******************************************************************
*
*  DESCRIPTION: class Transit
*
*  AUTHOR: Pablo Demidoff & Carlos Miracola 
*
*  EMAIL: mailto: pd4f@dc.uba.ar
*         mailto: cmiracol@dc.uba.ar
*
*  DATE: 26/04/2003
*
*******************************************************************/

/** include files **/
#include "transit.h"    // base class
#include "message.h"    // class ExternalMessage 
#include "mainsimu.h"   // class MainSimulator

/*******************************************************************
* Function Name: Transducer
********************************************************************/
Transit::Transit (const string &name)
: Atomic (name) 
, in (addInputPort ("in"))
, out (addOutputPort ("out")) {
	if (MainSimulator::Instance ().existsParameter (description (), "delay")) {
		delay = MainSimulator::Instance ().getParameter (description (), "delay");
	} else {
		// por default, Instantaneo.
		delay = Time::Zero;
	}
}

/*******************************************************************
* Function Name: initFunction
********************************************************************/
Model &Transit::initFunction () {
    unsolved.erase (unsolved.begin (), unsolved.end ());
	id = 0;
	
	passivate ();    
    return *this;
}

/*******************************************************************
* Function Name: externalFunction
********************************************************************/
Model &Transit::externalFunction (const ExternalMessage &msg) {
	Time e, next;
	int i;
    JobsList::iterator cursor;
	
	e = msg.time () - lastChange ();
	if (msg.port () == in) {
		// A todos les resto e !!!
		cursor = unsolved.begin ();
		while (cursor != unsolved.end ()) {
			i = (*cursor).first;
			unsolved [i] = unsolved [i] - e;
			cursor++;
		}
		// inserto el nuevo auto.
		if (unsolved.find (msg.value ()) != unsolved.end ()) {
			MException e (string ("Unresolved Work Id: ") + msg.value () + " is duplicated.");
			e.addLocation (MEXCEPTION_LOCATION ());
			throw e;
		}
		unsolved [msg.value ()] = delay;
		// calculo el tiempo en base al primer elemento..
		cursor = unsolved.begin ();
		id = (*cursor).first;
		next = (*cursor).second;		
		holdIn (active, next);
    }
    return *this;
}

/*******************************************************************
* Function Name: internalFunction
********************************************************************/
Model &Transit::internalFunction (const InternalMessage &msg) {
	Time e, next;
	int i;
    JobsList::iterator cursor;

	e = msg.time () - lastChange ();
	// elimino el primero...
	cursor = unsolved.begin ();
	i = (*cursor).first;
	unsolved.erase (i);
	
	// les resto a todos e !!!
	cursor = unsolved.begin ();
	while (cursor != unsolved.end ()) {
		i = (*cursor).first;
		unsolved [i] = unsolved [i] - e;
		cursor++;
	}

	// si tengo algo, lo scheduleo, sino, espero....
	if (!unsolved.empty ()) {
		cursor = unsolved.begin ();
		id = (*cursor).first;
		next = (*cursor).second;		
		holdIn (active, next);
	} else {
    	passivate ();
    }
    return *this;
}

/*******************************************************************
* Function Name: outputFunction
********************************************************************/
Model &Transit::outputFunction (const InternalMessage &msg) {
	sendOutput (msg.time (), out, id);
    return *this ;
}
